package com.creativewax.ribbons
{
	import flash.display.Sprite;
	import flash.geom.Point;

	public class Ribbon extends Sprite
	{
		public var particles:Vector.<RibbonParticle>	= new Vector.<RibbonParticle>();	// vector array that just holds RibbonParticles
		public var ribbonLayer:Sprite					= new Sprite();						// sprite to hold ribbon
		public var follower:Sprite;															// sprite to act as a pointer for the ribbon to follow
		
		public var maxAlpha:Number		= .5;						// maximum alpha when drawing the ribbon
		public var colour:Number;									// colour of the ribbon
		public var pAngle:Number		= Math.PI*.5;				// store 90 degress as radians in a variable so it does not have to be computed all the time
		
		public var wind:Number			= Math.random()*.5-.25;		// wind factor to effect the ribbons trail on the X axis
		public var gravity:Number		= Math.random()*.5-.25;		// gravty factor to effect the ribbons trail on the Y axis
		public var friction:Number;									// friction applied to the velocity of the ribbons movement
		public var tolerance:Number;								// tolerance in the elasticity of the ribbons trail
		public var maxParticles:Number;								// maximum amount of particles to be created in the ribbon
		public var maxDistance:Number;								// maximum distance that the particles can be apart
		public var growthRate:Number;								// growth rate dictates how thick the ribbon gets drawn
		
		private var __drawCommands:Vector.<int>		= new Vector.<int>(4, true);		// vector holding drawing commands
		private var __drawCoords:Vector.<Number>	= new Vector.<Number>(12, true);	// vector holding drawing coordinates
		
		private var __pt1:Point		= new Point();		// first point when working out drawing coordinates
		private var __pt2:Point		= new Point();		// second point when working out drawing coordinates
		private var __pt3:Point		= new Point();		// third point when working out drawing coordinates
		private var __pt4:Point		= new Point();		// fourth point when working out drawing coordinates
		private var __pt5:Point		= new Point();		// fifth point when working out drawing coordinates
		private var __pt6:Point		= new Point();		// sixth point when working out drawing coordinates
		
		private var __midPt1:Point	= new Point();		// first mid point when working out drawing coordinates
		private var __midPt2:Point	= new Point();		// second mid point when working out drawing coordinates
		private var __midPt3:Point	= new Point();		// third mid point when working out drawing coordinates
		private var __midPt4:Point	= new Point();		// fourth mid point when working out drawing coordinates
		
		// ______________________________________________
		// CONSTRUCTOR
		
		public function Ribbon(
			follower:Sprite,
			colour:Number			= 0xFFFFFF,
			maxParticles:int		= 20,
			maxDistance:Number		= 100,
			friction:Number			= .97,
			tolerance:Number		= .15,
			growthRate:Number		= 1
		)
		{
			this.follower		= follower;
			this.colour			= colour;
			this.maxParticles	= maxParticles;
			this.maxDistance	= maxDistance;
			this.friction		= friction;
			this.tolerance		= tolerance;
			this.growthRate		= growthRate;
			
			// init draw commands
			// these describe the way the draw graphics path command works
			__drawCommands[0] = 1;	// same as graphics.moveTo
			__drawCommands[1] = 3;	// same as graphics.curveTo
			__drawCommands[2] = 2;	// same as graphics.lineTo
			__drawCommands[3] = 3;	// same as graphics.curveTo
			
			// add to display
			addChild(ribbonLayer);
		}
		
		
		
		// ______________________________________________
		// PUBLIC
		
		public function createParticles(x:Number=0, y:Number=0):void
		{
			var r:Number	= 0;	// variable to hold radius width for the ribbon particle
			
			// loop through and create ribbon particles
			for (var i:uint=0; i<maxParticles; i++)
			{
				// set the radius of the particle to define the thickness of the ribbon
				r += growthRate;
				
				// create particle
				var particle:RibbonParticle	= new RibbonParticle(x, y, 0, 0, 0, r);
				
				// add to display
				ribbonLayer.addChild(particle);
				
				// add to main array holder
				particles.push(particle);
			}
		}
		
		// RENDERING METHODS
		public function render():void
		{
			moveParticles();	// move the particles
			drawRibbon();		// draw around the particles
		}
		
		
		
		// ______________________________________________
		// PRIVATE
		
		private function moveParticles():void
		{
			// get reference to the first particle
			var p:RibbonParticle	= particles[0];
			
			// update the particles position closer to the sprite the ribbon is following
			p.x		+= (follower.x - p.x)*.05;
			p.y		+= (follower.y - p.y)*.05;
			
			// loop through the particles to update their movement
			for (var i:uint=1; i<maxParticles; i++)
			{
				// get a reference to the current particle and the particle before it
				var p1:RibbonParticle	= particles[i];
				var p2:RibbonParticle	= particles[i-1];
				
				// add forces to velocity
				p1.velX		+= wind;
				p1.velY		+= gravity;
				
				// apply friction to velocity
				p1.velX		*= friction;
				p1.velY		*= friction;
				
				// add velocity to current position
				p1.x		+= p1.velX;
				p1.y		+= p1.velY;
				
				// check the distance between the two particles
				checkParticleDistance(p1, p2);
			}
		}
		
		private function drawRibbon():void
		{
			ribbonLayer.graphics.clear();
			
			// Draw line through all the points
			for (var i:uint=1; i<maxParticles-1; i++)
			{
				// get a reference to the current particle and its neighbours
				var p:RibbonParticle	= particles[i];
				var p2:RibbonParticle	= particles[i-1];
				var p3:RibbonParticle	= particles[i+1];
				
				// set the points around its radius, used to draw the curve
				setRadiusPoints(p3, __pt1, __pt2);
				setRadiusPoints(p, __pt3, __pt4);
				setRadiusPoints(p2, __pt5, __pt6);
				
				// work out mid points, these are also used for a smooth curve
				setMidPoints(__midPt1, __pt1, __pt3);
				setMidPoints(__midPt2, __pt2, __pt4);
				setMidPoints(__midPt3, __pt3, __pt5);
				setMidPoints(__midPt4, __pt4, __pt6);
				
				// populate the draw coordinates into our vector
			    __drawCoords[0] = __midPt2.x;
			    __drawCoords[1] = __midPt2.y;
			    __drawCoords[2] = __pt4.x;
			    __drawCoords[3] = __pt4.y;
			    __drawCoords[4] = __midPt4.x;
			    __drawCoords[5] = __midPt4.y;
			    __drawCoords[6] = __midPt3.x;
			    __drawCoords[7] = __midPt3.y;
			    __drawCoords[8] = __pt3.x;
			    __drawCoords[9] = __pt3.y;
			    __drawCoords[10] = __midPt1.x;
			    __drawCoords[11] = __midPt1.y;
			    
			    
			    // draw the shape we have just defined
			    ribbonLayer.graphics.beginFill(colour, maxAlpha - (maxAlpha/maxParticles)*i);
			    ribbonLayer.graphics.drawPath(__drawCommands, __drawCoords);
			    ribbonLayer.graphics.endFill();
			}
		}
		
		
		// set the position of the points that sit on the sides of the particle
		// these are set by the radius of the particle
		private function setRadiusPoints(p:RibbonParticle, pt1:Point, pt2:Point):void
		{
			pt1.x	= p.x + p.radius * Math.cos(p.angle + pAngle);
			pt1.y	= p.y + p.radius * Math.sin(p.angle + pAngle);
			pt2.x	= p.x - p.radius * Math.cos(p.angle + pAngle);
			pt2.y	= p.y - p.radius * Math.sin(p.angle + pAngle);
		}
		
		// get the mid point between the two points passed
		private function setMidPoints(midPt:Point, pt1:Point, pt2:Point):void
		{
			midPt.x	= pt1.x + (pt2.x - pt1.x)*.5;
			midPt.y	= pt1.y + (pt2.y - pt1.y)*.5;
		}
		
		private function checkParticleDistance(p:RibbonParticle, p2:RibbonParticle, b:Boolean=false):void
		{
			// get x and y distance between the two particles
			var dx:Number	= p.x - p2.x;
			var dy:Number	= p.y - p2.y;
			
			// calculate the actual distance between the particles
			p.dist			= Math.sqrt((dx * dx) + (dy * dy));
			
			// work out the angle of particle
			p2.angle		= Math.atan2(dy, dx);
			
			// if the distance stored in this particle is greater than the maximum
			// distance that we have set then start to drag the neighbouring particle
			if (p.dist > maxDistance)
			{
				// get a reference to the position the particle is
				var oldX:Number = p.x;
				var oldY:Number = p.y;
				
				// re-position the particle based on the position of the particle
				// it is following plus the maximum distance they can be apart
				p.x = p2.x + maxDistance * Math.cos(p2.angle);
				p.y = p2.y + maxDistance * Math.sin(p2.angle);
				
				// use the new and old positions plus the tolerance to
				// apply some more velocity to the particle
				p.velX	+= (p.x - oldX) * tolerance;
				p.velY	+= (p.y - oldY) * tolerance;
			}
		}
		
		
		
		// ______________________________________________
		// GETTERS / SETTERS
	}
}