package
{
	import com.creativewax.ribbons.Ribbon;
	
	import flash.display.*;
	import flash.events.Event;
	import flash.events.TimerEvent;
	import flash.utils.Timer;
	
	import gs.TweenMax;
	import gs.easing.*;

	public class CA_Ribbons extends Sprite
	{
		private var __holder:Sprite		= new Sprite();			// sprite that will hold the ribbons
		private var __follower:Sprite	= new Sprite();			// sprite that the ribbons will follow
		private var __timer:Timer		= new Timer(1000/30);	// timer to loop and render everything
		private var __friction:Number	= .6;					// friction for the follower
		
		private var __ribbons:Vector.<Ribbon>		= new Vector.<Ribbon>();	// vector holding all the ribbons we have created
		private var __total:uint					= 25;						// total amount of ribbons to create
		private var __wind:Number					= 0;						// wind factor to effect the ribbons trail on the X axis
		private var __gravity:Number				= 0;						// gravty factor to effect the ribbons trail on the Y axis
		private var __forceLimit:Number				= .002;						// amount of force to apply to the ribbons when rendering
		
		private var __sw:Number;			// stage width reference
		private var __sh:Number;			// stage height reference
		
		public function CA_Ribbons()
		{
			// setup stage and add a listener to deal with resizing the browser/swf
		 	stage.scaleMode		= StageScaleMode.NO_SCALE;
		 	stage.align			= StageAlign.TOP_LEFT;
			stage.quality		= StageQuality.HIGH;
			stage.displayState	= StageDisplayState.FULL_SCREEN;
			stage.addEventListener(Event.RESIZE, resize);
			
			resize();			// call resize method to set sizing variables to default values
			createRibbons();	// create all of our ribbons
			
			// setup and start timer
			__timer.addEventListener(TimerEvent.TIMER, render);
			__timer.start();
			
			// add holder sprite to display
			addChild(__holder);
		}
		
		// CREATE RIBBONS
		public function createRibbons():void
		{
			// loop and create ribbons
			for (var i:uint=0; i<__total; i++)
			{
				// create some random values to create our ribbons with
				var col:Number		= Math.random()*0xFFFFFF;	// colour
				var max:Number		= Math.random()*30+20;		// amount of particles that are used to make the ribbons
				var dist:Number		= Math.random()*50+50;		// distance betwen particles that are used to make the ribbons
				var friction:Number	= Math.random()*.03+.96;	// amount of friction between particles
				var tol:Number		= Math.random()*.3;			// tolerance amount when particles get dragged
				var growth:Number	= Math.random()*1+.2;		// growth rate of the ribbons
				
				// create ribbon
				var ribbon:Ribbon		= new Ribbon(__follower, col, max, dist, friction, tol, growth);
				ribbon.createParticles(__sw*.5, __sh*2);		// create the ribbons off stage
				
				// add ribbon to main array holder to retrive easier later on
				__ribbons.push(ribbon);
				
				// add ribbon to display
				__holder.addChild(ribbon);
			}
		}
		
		// RENDERING METHODS
		public function render(e:TimerEvent):void
		{
			moveCamera();		// update our fake camera movement
			updateRibbons();	// update ribbons movement
		}
		
		public function moveCamera():void
		{
			// work out the wind and gravity to effect the ribbons
			__wind		= (__follower.x - mouseX)*__forceLimit;
			__gravity	= (__follower.y - mouseY)*__forceLimit;
			
			// update the follower sprite position to the mouse position
			__follower.x	+= (mouseX - __follower.x) * __friction;
			__follower.y	+= (mouseY - __follower.y) * __friction;
		}
		
		public function updateRibbons():void
		{
			// loop through ribbons and update forces and render them
			for (var i:uint=0; i<__total; i++)
			{
				// get a reference to the ribbon we are effecting and update its wind and gravity and render it
				var ribbon:Ribbon	= __ribbons[i];
				ribbon.wind			= __wind;
				ribbon.gravity		= __gravity;	
				ribbon.render();
			}
		}
		
		// on stage resize update the variables for storing the stage width and height
		public function resize(e:Event=null):void
		{
			__sw	= stage.stageWidth;
			__sh	= stage.stageHeight;
		}
	}
}